# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/type/shared/v1/trade_data.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from systemathics.apis.type.shared.v1 import trade_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_trade__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1systemathics/apis/type/shared/v1/trade_data.proto\x12 systemathics.apis.type.shared.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,systemathics/apis/type/shared/v1/trade.proto\"\x84\x01\n\tTradeData\x12\x0f\n\x07mapping\x18\x01 \x01(\r\x12.\n\ntime_stamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x36\n\x05trade\x18\x03 \x01(\x0b\x32\'.systemathics.apis.type.shared.v1.Tradeb\x06proto3')



_TRADEDATA = DESCRIPTOR.message_types_by_name['TradeData']
TradeData = _reflection.GeneratedProtocolMessageType('TradeData', (_message.Message,), {
  'DESCRIPTOR' : _TRADEDATA,
  '__module__' : 'systemathics.apis.type.shared.v1.trade_data_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.type.shared.v1.TradeData)
  })
_sym_db.RegisterMessage(TradeData)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TRADEDATA._serialized_start=167
  _TRADEDATA._serialized_end=299
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/type/shared/v1/identifier_and_level.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import level_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_level__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;systemathics/apis/type/shared/v1/identifier_and_level.proto\x12 systemathics.apis.type.shared.v1\x1a\x1egoogle/protobuf/wrappers.proto\x1a,systemathics/apis/type/shared/v1/level.proto\"\x9e\x01\n\x12IdentifierAndLevel\x12\x10\n\x08\x65xchange\x18\x01 \x01(\t\x12\x0e\n\x06ticker\x18\x02 \x01(\t\x12.\n\x08provider\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x36\n\x05level\x18\x04 \x01(\x0e\x32\'.systemathics.apis.type.shared.v1.Levelb\x06proto3')



_IDENTIFIERANDLEVEL = DESCRIPTOR.message_types_by_name['IdentifierAndLevel']
IdentifierAndLevel = _reflection.GeneratedProtocolMessageType('IdentifierAndLevel', (_message.Message,), {
  'DESCRIPTOR' : _IDENTIFIERANDLEVEL,
  '__module__' : 'systemathics.apis.type.shared.v1.identifier_and_level_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.type.shared.v1.IdentifierAndLevel)
  })
_sym_db.RegisterMessage(IdentifierAndLevel)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _IDENTIFIERANDLEVEL._serialized_start=176
  _IDENTIFIERANDLEVEL._serialized_end=334
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/tick/v1/tick_trades.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import keys_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_keys__pb2
from systemathics.apis.type.shared.v1 import trade_data_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_trade__data__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4systemathics/apis/services/tick/v1/tick_trades.proto\x12\"systemathics.apis.services.tick.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a+systemathics/apis/type/shared/v1/keys.proto\x1a\x31systemathics/apis/type/shared/v1/trade_data.proto\"\xae\x01\n\x11TickTradesRequest\x12\x41\n\x0bidentifiers\x18\x01 \x03(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12\x42\n\x0b\x63onstraints\x18\x02 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x12\n\nadjustment\x18\x03 \x01(\x08\"\x97\x01\n\x12TickTradesResponse\x12;\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32+.systemathics.apis.type.shared.v1.TradeDataH\x00\x12\x39\n\x07mapping\x18\x02 \x01(\x0b\x32&.systemathics.apis.type.shared.v1.KeysH\x00\x42\t\n\x07payload2\x92\x01\n\x11TickTradesService\x12}\n\nTickTrades\x12\x35.systemathics.apis.services.tick.v1.TickTradesRequest\x1a\x36.systemathics.apis.services.tick.v1.TickTradesResponse0\x01\x62\x06proto3')



_TICKTRADESREQUEST = DESCRIPTOR.message_types_by_name['TickTradesRequest']
_TICKTRADESRESPONSE = DESCRIPTOR.message_types_by_name['TickTradesResponse']
TickTradesRequest = _reflection.GeneratedProtocolMessageType('TickTradesRequest', (_message.Message,), {
  'DESCRIPTOR' : _TICKTRADESREQUEST,
  '__module__' : 'systemathics.apis.services.tick.v1.tick_trades_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick.v1.TickTradesRequest)
  })
_sym_db.RegisterMessage(TickTradesRequest)

TickTradesResponse = _reflection.GeneratedProtocolMessageType('TickTradesResponse', (_message.Message,), {
  'DESCRIPTOR' : _TICKTRADESRESPONSE,
  '__module__' : 'systemathics.apis.services.tick.v1.tick_trades_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick.v1.TickTradesResponse)
  })
_sym_db.RegisterMessage(TickTradesResponse)

_TICKTRADESSERVICE = DESCRIPTOR.services_by_name['TickTradesService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TICKTRADESREQUEST._serialized_start=325
  _TICKTRADESREQUEST._serialized_end=499
  _TICKTRADESRESPONSE._serialized_start=502
  _TICKTRADESRESPONSE._serialized_end=653
  _TICKTRADESSERVICE._serialized_start=656
  _TICKTRADESSERVICE._serialized_end=802
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/intraday_analytics/v1/intraday_ema.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2
from systemathics.apis.type.shared.v1 import sampling_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_sampling__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nCsystemathics/apis/services/intraday_analytics/v1/intraday_ema.proto\x12\x30systemathics.apis.services.intraday_analytics.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\x1a/systemathics/apis/type/shared/v1/sampling.proto\"\xfc\x01\n\x12IntradayEmaRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12<\n\x08sampling\x18\x02 \x01(\x0e\x32*.systemathics.apis.type.shared.v1.Sampling\x12\x42\n\x0b\x63onstraints\x18\x03 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x0e\n\x06length\x18\x04 \x01(\x05\x12\x12\n\nadjustment\x18\x05 \x01(\x08\"f\n\x13IntradayEmaResponse\x12O\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x41.systemathics.apis.services.intraday_analytics.v1.IntradayEmaData\"a\n\x0fIntradayEmaData\x12.\n\ntime_stamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x0f\n\x07\x61verage\x18\x03 \x01(\x01\x32\xb1\x01\n\x12IntradayEmaService\x12\x9a\x01\n\x0bIntradayEma\x12\x44.systemathics.apis.services.intraday_analytics.v1.IntradayEmaRequest\x1a\x45.systemathics.apis.services.intraday_analytics.v1.IntradayEmaResponseb\x06proto3')



_INTRADAYEMAREQUEST = DESCRIPTOR.message_types_by_name['IntradayEmaRequest']
_INTRADAYEMARESPONSE = DESCRIPTOR.message_types_by_name['IntradayEmaResponse']
_INTRADAYEMADATA = DESCRIPTOR.message_types_by_name['IntradayEmaData']
IntradayEmaRequest = _reflection.GeneratedProtocolMessageType('IntradayEmaRequest', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYEMAREQUEST,
  '__module__' : 'systemathics.apis.services.intraday_analytics.v1.intraday_ema_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday_analytics.v1.IntradayEmaRequest)
  })
_sym_db.RegisterMessage(IntradayEmaRequest)

IntradayEmaResponse = _reflection.GeneratedProtocolMessageType('IntradayEmaResponse', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYEMARESPONSE,
  '__module__' : 'systemathics.apis.services.intraday_analytics.v1.intraday_ema_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday_analytics.v1.IntradayEmaResponse)
  })
_sym_db.RegisterMessage(IntradayEmaResponse)

IntradayEmaData = _reflection.GeneratedProtocolMessageType('IntradayEmaData', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYEMADATA,
  '__module__' : 'systemathics.apis.services.intraday_analytics.v1.intraday_ema_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday_analytics.v1.IntradayEmaData)
  })
_sym_db.RegisterMessage(IntradayEmaData)

_INTRADAYEMASERVICE = DESCRIPTOR.services_by_name['IntradayEmaService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INTRADAYEMAREQUEST._serialized_start=339
  _INTRADAYEMAREQUEST._serialized_end=591
  _INTRADAYEMARESPONSE._serialized_start=593
  _INTRADAYEMARESPONSE._serialized_end=695
  _INTRADAYEMADATA._serialized_start=697
  _INTRADAYEMADATA._serialized_end=794
  _INTRADAYEMASERVICE._serialized_start=797
  _INTRADAYEMASERVICE._serialized_end=974
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/daily_analytics/v1/daily_sma.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.type import date_pb2 as google_dot_type_dot_date__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=systemathics/apis/services/daily_analytics/v1/daily_sma.proto\x12-systemathics.apis.services.daily_analytics.v1\x1a\x16google/type/date.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\"\xbb\x01\n\x0f\x44\x61ilySmaRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12\x42\n\x0b\x63onstraints\x18\x02 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x0e\n\x06length\x18\x03 \x01(\x05\x12\x12\n\nadjustment\x18\x04 \x01(\x08\"]\n\x10\x44\x61ilySmaResponse\x12I\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32;.systemathics.apis.services.daily_analytics.v1.DailySmaData\"m\n\x0c\x44\x61ilySmaData\x12\x1f\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.Date\x12\r\n\x05value\x18\x02 \x01(\x01\x12-\n\x07\x61verage\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue2\x9f\x01\n\x0f\x44\x61ilySmaService\x12\x8b\x01\n\x08\x44\x61ilySma\x12>.systemathics.apis.services.daily_analytics.v1.DailySmaRequest\x1a?.systemathics.apis.services.daily_analytics.v1.DailySmaResponseb\x06proto3')



_DAILYSMAREQUEST = DESCRIPTOR.message_types_by_name['DailySmaRequest']
_DAILYSMARESPONSE = DESCRIPTOR.message_types_by_name['DailySmaResponse']
_DAILYSMADATA = DESCRIPTOR.message_types_by_name['DailySmaData']
DailySmaRequest = _reflection.GeneratedProtocolMessageType('DailySmaRequest', (_message.Message,), {
  'DESCRIPTOR' : _DAILYSMAREQUEST,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_sma_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailySmaRequest)
  })
_sym_db.RegisterMessage(DailySmaRequest)

DailySmaResponse = _reflection.GeneratedProtocolMessageType('DailySmaResponse', (_message.Message,), {
  'DESCRIPTOR' : _DAILYSMARESPONSE,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_sma_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailySmaResponse)
  })
_sym_db.RegisterMessage(DailySmaResponse)

DailySmaData = _reflection.GeneratedProtocolMessageType('DailySmaData', (_message.Message,), {
  'DESCRIPTOR' : _DAILYSMADATA,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_sma_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailySmaData)
  })
_sym_db.RegisterMessage(DailySmaData)

_DAILYSMASERVICE = DESCRIPTOR.services_by_name['DailySmaService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DAILYSMAREQUEST._serialized_start=272
  _DAILYSMAREQUEST._serialized_end=459
  _DAILYSMARESPONSE._serialized_start=461
  _DAILYSMARESPONSE._serialized_end=554
  _DAILYSMADATA._serialized_start=556
  _DAILYSMADATA._serialized_end=665
  _DAILYSMASERVICE._serialized_start=668
  _DAILYSMASERVICE._serialized_end=827
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/tick/v1/tick_trades_and_book.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2
from systemathics.apis.type.shared.v1 import identifier_and_level_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__and__level__pb2
from systemathics.apis.type.shared.v1 import keys_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_keys__pb2
from systemathics.apis.type.shared.v1 import book_updates_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_book__updates__pb2
from systemathics.apis.type.shared.v1 import trade_and_book_data_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_trade__and__book__data__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=systemathics/apis/services/tick/v1/tick_trades_and_book.proto\x12\"systemathics.apis.services.tick.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\x1a;systemathics/apis/type/shared/v1/identifier_and_level.proto\x1a+systemathics/apis/type/shared/v1/keys.proto\x1a\x33systemathics/apis/type/shared/v1/book_updates.proto\x1a:systemathics/apis/type/shared/v1/trade_and_book_data.proto\"\xb2\x02\n\x18TickTradesAndBookRequest\x12I\n\x0bidentifiers\x18\x01 \x03(\x0b\x32\x34.systemathics.apis.type.shared.v1.IdentifierAndLevel\x12\x43\n\x0c\x62ook_updates\x18\x02 \x01(\x0e\x32-.systemathics.apis.type.shared.v1.BookUpdates\x12\x42\n\x0b\x63onstraints\x18\x03 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x12\n\nadjustment\x18\x04 \x01(\x08\x12.\n\tmax_depth\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\"\xa5\x01\n\x19TickTradesAndBookResponse\x12\x42\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x32.systemathics.apis.type.shared.v1.TradeAndBookDataH\x00\x12\x39\n\x07mapping\x18\x02 \x01(\x0b\x32&.systemathics.apis.type.shared.v1.KeysH\x00\x42\t\n\x07payload2\xaf\x01\n\x18TickTradesAndBookService\x12\x92\x01\n\x11TickTradesAndBook\x12<.systemathics.apis.services.tick.v1.TickTradesAndBookRequest\x1a=.systemathics.apis.services.tick.v1.TickTradesAndBookResponse0\x01\x62\x06proto3')



_TICKTRADESANDBOOKREQUEST = DESCRIPTOR.message_types_by_name['TickTradesAndBookRequest']
_TICKTRADESANDBOOKRESPONSE = DESCRIPTOR.message_types_by_name['TickTradesAndBookResponse']
TickTradesAndBookRequest = _reflection.GeneratedProtocolMessageType('TickTradesAndBookRequest', (_message.Message,), {
  'DESCRIPTOR' : _TICKTRADESANDBOOKREQUEST,
  '__module__' : 'systemathics.apis.services.tick.v1.tick_trades_and_book_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick.v1.TickTradesAndBookRequest)
  })
_sym_db.RegisterMessage(TickTradesAndBookRequest)

TickTradesAndBookResponse = _reflection.GeneratedProtocolMessageType('TickTradesAndBookResponse', (_message.Message,), {
  'DESCRIPTOR' : _TICKTRADESANDBOOKRESPONSE,
  '__module__' : 'systemathics.apis.services.tick.v1.tick_trades_and_book_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick.v1.TickTradesAndBookResponse)
  })
_sym_db.RegisterMessage(TickTradesAndBookResponse)

_TICKTRADESANDBOOKSERVICE = DESCRIPTOR.services_by_name['TickTradesAndBookService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TICKTRADESANDBOOKREQUEST._serialized_start=438
  _TICKTRADESANDBOOKREQUEST._serialized_end=744
  _TICKTRADESANDBOOKRESPONSE._serialized_start=747
  _TICKTRADESANDBOOKRESPONSE._serialized_end=912
  _TICKTRADESANDBOOKSERVICE._serialized_start=915
  _TICKTRADESANDBOOKSERVICE._serialized_end=1090
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/daily_analytics/v1/daily_cma.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.type import date_pb2 as google_dot_type_dot_date__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=systemathics/apis/services/daily_analytics/v1/daily_cma.proto\x12-systemathics.apis.services.daily_analytics.v1\x1a\x16google/type/date.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\"\xab\x01\n\x0f\x44\x61ilyCmaRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12\x42\n\x0b\x63onstraints\x18\x02 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x12\n\nadjustment\x18\x03 \x01(\x08\"]\n\x10\x44\x61ilyCmaResponse\x12I\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32;.systemathics.apis.services.daily_analytics.v1.DailyCmaData\"O\n\x0c\x44\x61ilyCmaData\x12\x1f\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.Date\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x0f\n\x07\x61verage\x18\x03 \x01(\x01\x32\x9f\x01\n\x0f\x44\x61ilyCmaService\x12\x8b\x01\n\x08\x44\x61ilyCma\x12>.systemathics.apis.services.daily_analytics.v1.DailyCmaRequest\x1a?.systemathics.apis.services.daily_analytics.v1.DailyCmaResponseb\x06proto3')



_DAILYCMAREQUEST = DESCRIPTOR.message_types_by_name['DailyCmaRequest']
_DAILYCMARESPONSE = DESCRIPTOR.message_types_by_name['DailyCmaResponse']
_DAILYCMADATA = DESCRIPTOR.message_types_by_name['DailyCmaData']
DailyCmaRequest = _reflection.GeneratedProtocolMessageType('DailyCmaRequest', (_message.Message,), {
  'DESCRIPTOR' : _DAILYCMAREQUEST,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_cma_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyCmaRequest)
  })
_sym_db.RegisterMessage(DailyCmaRequest)

DailyCmaResponse = _reflection.GeneratedProtocolMessageType('DailyCmaResponse', (_message.Message,), {
  'DESCRIPTOR' : _DAILYCMARESPONSE,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_cma_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyCmaResponse)
  })
_sym_db.RegisterMessage(DailyCmaResponse)

DailyCmaData = _reflection.GeneratedProtocolMessageType('DailyCmaData', (_message.Message,), {
  'DESCRIPTOR' : _DAILYCMADATA,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_cma_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyCmaData)
  })
_sym_db.RegisterMessage(DailyCmaData)

_DAILYCMASERVICE = DESCRIPTOR.services_by_name['DailyCmaService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DAILYCMAREQUEST._serialized_start=272
  _DAILYCMAREQUEST._serialized_end=443
  _DAILYCMARESPONSE._serialized_start=445
  _DAILYCMARESPONSE._serialized_end=538
  _DAILYCMADATA._serialized_start=540
  _DAILYCMADATA._serialized_end=619
  _DAILYCMASERVICE._serialized_start=622
  _DAILYCMASERVICE._serialized_end=781
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/tick/v1/tick_quotes.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import keys_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_keys__pb2
from systemathics.apis.type.shared.v1 import quotes_data_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_quotes__data__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4systemathics/apis/services/tick/v1/tick_quotes.proto\x12\"systemathics.apis.services.tick.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a+systemathics/apis/type/shared/v1/keys.proto\x1a\x32systemathics/apis/type/shared/v1/quotes_data.proto\"\xae\x01\n\x11TickQuotesRequest\x12\x41\n\x0bidentifiers\x18\x01 \x03(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12\x42\n\x0b\x63onstraints\x18\x02 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x12\n\nadjustment\x18\x03 \x01(\x08\"\x98\x01\n\x12TickQuotesResponse\x12<\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.QuotesDataH\x00\x12\x39\n\x07mapping\x18\x02 \x01(\x0b\x32&.systemathics.apis.type.shared.v1.KeysH\x00\x42\t\n\x07payload2\x92\x01\n\x11TickQuotesService\x12}\n\nTickQuotes\x12\x35.systemathics.apis.services.tick.v1.TickQuotesRequest\x1a\x36.systemathics.apis.services.tick.v1.TickQuotesResponse0\x01\x62\x06proto3')



_TICKQUOTESREQUEST = DESCRIPTOR.message_types_by_name['TickQuotesRequest']
_TICKQUOTESRESPONSE = DESCRIPTOR.message_types_by_name['TickQuotesResponse']
TickQuotesRequest = _reflection.GeneratedProtocolMessageType('TickQuotesRequest', (_message.Message,), {
  'DESCRIPTOR' : _TICKQUOTESREQUEST,
  '__module__' : 'systemathics.apis.services.tick.v1.tick_quotes_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick.v1.TickQuotesRequest)
  })
_sym_db.RegisterMessage(TickQuotesRequest)

TickQuotesResponse = _reflection.GeneratedProtocolMessageType('TickQuotesResponse', (_message.Message,), {
  'DESCRIPTOR' : _TICKQUOTESRESPONSE,
  '__module__' : 'systemathics.apis.services.tick.v1.tick_quotes_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick.v1.TickQuotesResponse)
  })
_sym_db.RegisterMessage(TickQuotesResponse)

_TICKQUOTESSERVICE = DESCRIPTOR.services_by_name['TickQuotesService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TICKQUOTESREQUEST._serialized_start=358
  _TICKQUOTESREQUEST._serialized_end=532
  _TICKQUOTESRESPONSE._serialized_start=535
  _TICKQUOTESRESPONSE._serialized_end=687
  _TICKQUOTESSERVICE._serialized_start=690
  _TICKQUOTESSERVICE._serialized_end=836
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/type/shared/v1/book_updates.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3systemathics/apis/type/shared/v1/book_updates.proto\x12 systemathics.apis.type.shared.v1*\x88\x01\n\x0b\x42ookUpdates\x12\x1c\n\x18\x42OOK_UPDATES_UNSPECIFIED\x10\x00\x12\x1f\n\x1b\x42OOK_UPDATES_SNAPSHOTS_ONLY\x10\x01\x12:\n6BOOK_UPDATES_INITIAL_SNAPSHOT_THEN_INCREMENTAL_UPDATES\x10\x02\x62\x06proto3')

_BOOKUPDATES = DESCRIPTOR.enum_types_by_name['BookUpdates']
BookUpdates = enum_type_wrapper.EnumTypeWrapper(_BOOKUPDATES)
BOOK_UPDATES_UNSPECIFIED = 0
BOOK_UPDATES_SNAPSHOTS_ONLY = 1
BOOK_UPDATES_INITIAL_SNAPSHOT_THEN_INCREMENTAL_UPDATES = 2


if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BOOKUPDATES._serialized_start=90
  _BOOKUPDATES._serialized_end=226
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/type/shared/v1/time_interval.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.type import timeofday_pb2 as google_dot_type_dot_timeofday__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4systemathics/apis/type/shared/v1/time_interval.proto\x12 systemathics.apis.type.shared.v1\x1a\x1bgoogle/type/timeofday.proto\"d\n\x0cTimeInterval\x12*\n\nstart_time\x18\x01 \x01(\x0b\x32\x16.google.type.TimeOfDay\x12(\n\x08\x65nd_time\x18\x02 \x01(\x0b\x32\x16.google.type.TimeOfDayb\x06proto3')



_TIMEINTERVAL = DESCRIPTOR.message_types_by_name['TimeInterval']
TimeInterval = _reflection.GeneratedProtocolMessageType('TimeInterval', (_message.Message,), {
  'DESCRIPTOR' : _TIMEINTERVAL,
  '__module__' : 'systemathics.apis.type.shared.v1.time_interval_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.type.shared.v1.TimeInterval)
  })
_sym_db.RegisterMessage(TimeInterval)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TIMEINTERVAL._serialized_start=119
  _TIMEINTERVAL._serialized_end=219
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/tick_analytics/v1/tick_vwap.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<systemathics/apis/services/tick_analytics/v1/tick_vwap.proto\x12,systemathics.apis.services.tick_analytics.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\"\xa1\x02\n\x0fTickVwapRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12\x42\n\x0b\x63onstraints\x18\x02 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12)\n\x06period\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12)\n\x06offset\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x12\n\nadjustment\x18\x05 \x01(\x08\x12\x0e\n\x06\x61\x63\x63\x65pt\x18\x06 \x03(\t\x12\x0e\n\x06reject\x18\x07 \x03(\t\"~\n\x10TickVwapResponse\x12.\n\ntime_stamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04vwap\x18\x02 \x01(\x01\x12\r\n\x05trade\x18\x03 \x01(\x01\x12\x0e\n\x06volume\x18\x04 \x01(\x03\x12\r\n\x05ticks\x18\x05 \x01(\x05\x32\x9f\x01\n\x0fTickVwapService\x12\x8b\x01\n\x08TickVwap\x12=.systemathics.apis.services.tick_analytics.v1.TickVwapRequest\x1a>.systemathics.apis.services.tick_analytics.v1.TickVwapResponse0\x01\x62\x06proto3')



_TICKVWAPREQUEST = DESCRIPTOR.message_types_by_name['TickVwapRequest']
_TICKVWAPRESPONSE = DESCRIPTOR.message_types_by_name['TickVwapResponse']
TickVwapRequest = _reflection.GeneratedProtocolMessageType('TickVwapRequest', (_message.Message,), {
  'DESCRIPTOR' : _TICKVWAPREQUEST,
  '__module__' : 'systemathics.apis.services.tick_analytics.v1.tick_vwap_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick_analytics.v1.TickVwapRequest)
  })
_sym_db.RegisterMessage(TickVwapRequest)

TickVwapResponse = _reflection.GeneratedProtocolMessageType('TickVwapResponse', (_message.Message,), {
  'DESCRIPTOR' : _TICKVWAPRESPONSE,
  '__module__' : 'systemathics.apis.services.tick_analytics.v1.tick_vwap_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.tick_analytics.v1.TickVwapResponse)
  })
_sym_db.RegisterMessage(TickVwapResponse)

_TICKVWAPSERVICE = DESCRIPTOR.services_by_name['TickVwapService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TICKVWAPREQUEST._serialized_start=279
  _TICKVWAPREQUEST._serialized_end=568
  _TICKVWAPRESPONSE._serialized_start=570
  _TICKVWAPRESPONSE._serialized_end=696
  _TICKVWAPSERVICE._serialized_start=699
  _TICKVWAPSERVICE._serialized_end=858
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/daily_analytics/v1/daily_ema.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.type import date_pb2 as google_dot_type_dot_date__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=systemathics/apis/services/daily_analytics/v1/daily_ema.proto\x12-systemathics.apis.services.daily_analytics.v1\x1a\x16google/type/date.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\"\xbb\x01\n\x0f\x44\x61ilyEmaRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12\x42\n\x0b\x63onstraints\x18\x02 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x0e\n\x06length\x18\x03 \x01(\x05\x12\x12\n\nadjustment\x18\x04 \x01(\x08\"]\n\x10\x44\x61ilyEmaResponse\x12I\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32;.systemathics.apis.services.daily_analytics.v1.DailyEmaData\"O\n\x0c\x44\x61ilyEmaData\x12\x1f\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.Date\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x0f\n\x07\x61verage\x18\x03 \x01(\x01\x32\x9f\x01\n\x0f\x44\x61ilyEmaService\x12\x8b\x01\n\x08\x44\x61ilyEma\x12>.systemathics.apis.services.daily_analytics.v1.DailyEmaRequest\x1a?.systemathics.apis.services.daily_analytics.v1.DailyEmaResponseb\x06proto3')



_DAILYEMAREQUEST = DESCRIPTOR.message_types_by_name['DailyEmaRequest']
_DAILYEMARESPONSE = DESCRIPTOR.message_types_by_name['DailyEmaResponse']
_DAILYEMADATA = DESCRIPTOR.message_types_by_name['DailyEmaData']
DailyEmaRequest = _reflection.GeneratedProtocolMessageType('DailyEmaRequest', (_message.Message,), {
  'DESCRIPTOR' : _DAILYEMAREQUEST,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_ema_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyEmaRequest)
  })
_sym_db.RegisterMessage(DailyEmaRequest)

DailyEmaResponse = _reflection.GeneratedProtocolMessageType('DailyEmaResponse', (_message.Message,), {
  'DESCRIPTOR' : _DAILYEMARESPONSE,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_ema_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyEmaResponse)
  })
_sym_db.RegisterMessage(DailyEmaResponse)

DailyEmaData = _reflection.GeneratedProtocolMessageType('DailyEmaData', (_message.Message,), {
  'DESCRIPTOR' : _DAILYEMADATA,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_ema_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyEmaData)
  })
_sym_db.RegisterMessage(DailyEmaData)

_DAILYEMASERVICE = DESCRIPTOR.services_by_name['DailyEmaService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DAILYEMAREQUEST._serialized_start=272
  _DAILYEMAREQUEST._serialized_end=459
  _DAILYEMARESPONSE._serialized_start=461
  _DAILYEMARESPONSE._serialized_end=554
  _DAILYEMADATA._serialized_start=556
  _DAILYEMADATA._serialized_end=635
  _DAILYEMASERVICE._serialized_start=638
  _DAILYEMASERVICE._serialized_end=797
# @@protoc_insertion_point(module_scope)



"""Systemathics Ganymede  APIs Token Helpers

This module helps to create tokens to access Systemathics Ganymede authenticated APIs.

functions:
    * get_token - Get a JWT Authorization token suitable to call Ganymede gRPC APIs.
"""

import os
import http.client
import json

DEFAULT_AUDIENCE = "https://prod.ganymede-prod"

DEFAULT_TENANT = "ganymede-prod.eu.auth0.com"

def get_token_as_metadata() -> []:
    """
    Get a JWT Authorization token suitable to call Ganymede gRPC APIs.
    We either use 'AUTH0_TOKEN' environment variable (if present) to create a bearer token from it.
    Or 'CLIENT_ID' and 'CLIENT_SECRET' environment variables (optionally 'AUDIENCE' can override DEFAULT_AUDIENCE, and 'TENANT' can override DEFAULT_TENANT).  
    Returns:
        A JWT Authorization token suitable to call Ganymede gRPC APIs in a form directly assignable to metadata= in stub call, that is [('authorization', get_token())].
    """
    return [('authorization', get_token())]

def get_token() -> str:    
    """
    Get a JWT Authorization token suitable to call Ganymede gRPC APIs.
    We either use 'AUTH0_TOKEN' environment variable (if present) to create a bearer token from it.
    Or 'CLIENT_ID' and 'CLIENT_SECRET' environment variables (optionally 'AUDIENCE' can override DEFAULT_AUDIENCE, and 'TENANT' can override DEFAULT_TENANT).  
    Returns:
        A JWT Authorization token suitable to call Ganymede gRPC APIs.
    """
    auth0_token = os.getenv("AUTH0_TOKEN","")
    client_id = os.getenv("CLIENT_ID","")
    client_secret = os.getenv("CLIENT_SECRET","")
    audience = os.getenv("AUDIENCE","")
    tenant = os.getenv("TENANT","")

    # If we have AUTH0_TOKEN, generate a bearer token
    if(auth0_token != ""):
        return f"Bearer {auth0_token}"

    # If we don't, use Auth0 REST API to request one (we need CLIENT_ID and CLIENT_SECRET; Optionally AUDIENCE and TENANT)
    if (client_id and client_secret):
        return _create_bearer_token_using_rest(
            client_id, 
            client_secret, 
            audience if audience else DEFAULT_AUDIENCE, 
            tenant if tenant else DEFAULT_TENANT)
    else:    
        raise Exception(f"AUTH0_TOKEN environment variable is not set, therefore CLIENT_ID and CLIENT_SECRET (and optionally AUDIENCE and TENANT) environment variables must be set")

def _create_bearer_token_using_rest(client_id, client_secret, audience, tenant) -> str:
    if (client_id == ""):
        raise Exception(f"client_id cannot be null")
    if (client_secret == ""):
        raise Exception(f"client_secret cannot be null")
    if (audience == ""):
        raise Exception(f"audience cannot be null")
    if (tenant == ""):
        raise Exception(f"tenant cannot be null")

    # Setup connection and payload
    conn = http.client.HTTPSConnection(tenant)
    headers = { "content-type": "application/json" }
    params = {"client_id": client_id, "client_secret": client_secret, "grant_type" : "client_credentials", "audience": audience }
    payload = json.dumps(params)

    # Send Request
    conn.request("POST", "/oauth/token", payload, headers)
    res = conn.getresponse()
    data = res.read()
       
    json_data =  json.loads(data.decode("utf-8"))
                            
    # Get access token to be used to authenticate against API
    try:
        token = f"{json_data['token_type']} {json_data['access_token']}"
        os.environ['AUTH0_TOKEN'] = json_data['access_token'] # Push to environment
        return token
    except Exception as ee:
        raise Exception(f"Returned JSON doesn't contain 'token_type' and/or 'access_token'. Check your client ID, client secret, audience and tenant: {json_data}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/intraday/v1/intraday_bars.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2
from systemathics.apis.type.shared.v1 import sampling_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_sampling__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n:systemathics/apis/services/intraday/v1/intraday_bars.proto\x12&systemathics.apis.services.intraday.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\x1a/systemathics/apis/type/shared/v1/sampling.proto\"\xed\x01\n\x13IntradayBarsRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12<\n\x08sampling\x18\x02 \x01(\x0e\x32*.systemathics.apis.type.shared.v1.Sampling\x12\x42\n\x0b\x63onstraints\x18\x03 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x12\n\nadjustment\x18\x04 \x01(\x08\"Y\n\x14IntradayBarsResponse\x12\x41\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x33.systemathics.apis.services.intraday.v1.IntradayBar\"\xb1\x01\n\x0bIntradayBar\x12.\n\ntime_stamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04open\x18\x02 \x01(\x01\x12\x0c\n\x04high\x18\x03 \x01(\x01\x12\x0b\n\x03low\x18\x04 \x01(\x01\x12\r\n\x05\x63lose\x18\x05 \x01(\x01\x12\x0e\n\x06volume\x18\x06 \x01(\x03\x12\r\n\x05\x63ount\x18\x07 \x01(\x05\x12\x0c\n\x04vwap\x18\x08 \x01(\x01\x12\r\n\x05score\x18\t \x01(\x01\x32\xa1\x01\n\x13IntradayBarsService\x12\x89\x01\n\x0cIntradayBars\x12;.systemathics.apis.services.intraday.v1.IntradayBarsRequest\x1a<.systemathics.apis.services.intraday.v1.IntradayBarsResponseb\x06proto3')



_INTRADAYBARSREQUEST = DESCRIPTOR.message_types_by_name['IntradayBarsRequest']
_INTRADAYBARSRESPONSE = DESCRIPTOR.message_types_by_name['IntradayBarsResponse']
_INTRADAYBAR = DESCRIPTOR.message_types_by_name['IntradayBar']
IntradayBarsRequest = _reflection.GeneratedProtocolMessageType('IntradayBarsRequest', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYBARSREQUEST,
  '__module__' : 'systemathics.apis.services.intraday.v1.intraday_bars_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday.v1.IntradayBarsRequest)
  })
_sym_db.RegisterMessage(IntradayBarsRequest)

IntradayBarsResponse = _reflection.GeneratedProtocolMessageType('IntradayBarsResponse', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYBARSRESPONSE,
  '__module__' : 'systemathics.apis.services.intraday.v1.intraday_bars_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday.v1.IntradayBarsResponse)
  })
_sym_db.RegisterMessage(IntradayBarsResponse)

IntradayBar = _reflection.GeneratedProtocolMessageType('IntradayBar', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYBAR,
  '__module__' : 'systemathics.apis.services.intraday.v1.intraday_bars_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday.v1.IntradayBar)
  })
_sym_db.RegisterMessage(IntradayBar)

_INTRADAYBARSSERVICE = DESCRIPTOR.services_by_name['IntradayBarsService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INTRADAYBARSREQUEST._serialized_start=288
  _INTRADAYBARSREQUEST._serialized_end=525
  _INTRADAYBARSRESPONSE._serialized_start=527
  _INTRADAYBARSRESPONSE._serialized_end=616
  _INTRADAYBAR._serialized_start=619
  _INTRADAYBAR._serialized_end=796
  _INTRADAYBARSSERVICE._serialized_start=799
  _INTRADAYBARSSERVICE._serialized_end=960
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/daily_analytics/v1/daily_macd.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.type import date_pb2 as google_dot_type_dot_date__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n>systemathics/apis/services/daily_analytics/v1/daily_macd.proto\x12-systemathics.apis.services.daily_analytics.v1\x1a\x16google/type/date.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\"\xc9\x01\n\x10\x44\x61ilyMacdRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12\x42\n\x0b\x63onstraints\x18\x02 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x0c\n\x04long\x18\x03 \x01(\x05\x12\r\n\x05short\x18\x04 \x01(\x05\x12\x12\n\nadjustment\x18\x05 \x01(\x08\"_\n\x11\x44\x61ilyMacdResponse\x12J\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32<.systemathics.apis.services.daily_analytics.v1.DailyMacdData\"j\n\rDailyMacdData\x12\x1f\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.Date\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x0c\n\x04macd\x18\x03 \x01(\x01\x12\r\n\x05short\x18\x04 \x01(\x01\x12\x0c\n\x04long\x18\x05 \x01(\x01\x32\xa3\x01\n\x10\x44\x61ilyMacdService\x12\x8e\x01\n\tDailyMacd\x12?.systemathics.apis.services.daily_analytics.v1.DailyMacdRequest\x1a@.systemathics.apis.services.daily_analytics.v1.DailyMacdResponseb\x06proto3')



_DAILYMACDREQUEST = DESCRIPTOR.message_types_by_name['DailyMacdRequest']
_DAILYMACDRESPONSE = DESCRIPTOR.message_types_by_name['DailyMacdResponse']
_DAILYMACDDATA = DESCRIPTOR.message_types_by_name['DailyMacdData']
DailyMacdRequest = _reflection.GeneratedProtocolMessageType('DailyMacdRequest', (_message.Message,), {
  'DESCRIPTOR' : _DAILYMACDREQUEST,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_macd_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyMacdRequest)
  })
_sym_db.RegisterMessage(DailyMacdRequest)

DailyMacdResponse = _reflection.GeneratedProtocolMessageType('DailyMacdResponse', (_message.Message,), {
  'DESCRIPTOR' : _DAILYMACDRESPONSE,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_macd_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyMacdResponse)
  })
_sym_db.RegisterMessage(DailyMacdResponse)

DailyMacdData = _reflection.GeneratedProtocolMessageType('DailyMacdData', (_message.Message,), {
  'DESCRIPTOR' : _DAILYMACDDATA,
  '__module__' : 'systemathics.apis.services.daily_analytics.v1.daily_macd_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.daily_analytics.v1.DailyMacdData)
  })
_sym_db.RegisterMessage(DailyMacdData)

_DAILYMACDSERVICE = DESCRIPTOR.services_by_name['DailyMacdService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DAILYMACDREQUEST._serialized_start=241
  _DAILYMACDREQUEST._serialized_end=442
  _DAILYMACDRESPONSE._serialized_start=444
  _DAILYMACDRESPONSE._serialized_end=539
  _DAILYMACDDATA._serialized_start=541
  _DAILYMACDDATA._serialized_end=647
  _DAILYMACDSERVICE._serialized_start=650
  _DAILYMACDSERVICE._serialized_end=813
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/type/shared/v1/action.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-systemathics/apis/type/shared/v1/action.proto\x12 systemathics.apis.type.shared.v1*B\n\x06\x41\x63tion\x12\x16\n\x12\x41\x43TION_UNSPECIFIED\x10\x00\x12\x0e\n\nACTION_SET\x10\x01\x12\x10\n\x0c\x41\x43TION_CLEAR\x10\x02\x62\x06proto3')

_ACTION = DESCRIPTOR.enum_types_by_name['Action']
Action = enum_type_wrapper.EnumTypeWrapper(_ACTION)
ACTION_UNSPECIFIED = 0
ACTION_SET = 1
ACTION_CLEAR = 2


if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ACTION._serialized_start=83
  _ACTION._serialized_end=149
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/intraday_analytics/v1/intraday_bollinger.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import constraints_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_constraints__pb2
from systemathics.apis.type.shared.v1 import sampling_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_sampling__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nIsystemathics/apis/services/intraday_analytics/v1/intraday_bollinger.proto\x12\x30systemathics.apis.services.intraday_analytics.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x32systemathics/apis/type/shared/v1/constraints.proto\x1a/systemathics/apis/type/shared/v1/sampling.proto\"\x95\x02\n\x18IntradayBollingerRequest\x12@\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.Identifier\x12<\n\x08sampling\x18\x02 \x01(\x0e\x32*.systemathics.apis.type.shared.v1.Sampling\x12\x42\n\x0b\x63onstraints\x18\x03 \x01(\x0b\x32-.systemathics.apis.type.shared.v1.Constraints\x12\x0e\n\x06length\x18\x04 \x01(\x05\x12\x11\n\tdeviation\x18\x05 \x01(\x01\x12\x12\n\nadjustment\x18\x06 \x01(\x08\"r\n\x19IntradayBollingerResponse\x12U\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32G.systemathics.apis.services.intraday_analytics.v1.IntradayBollingerData\"\xde\x01\n\x15IntradayBollingerData\x12.\n\ntime_stamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05value\x18\x02 \x01(\x01\x12+\n\x05lower\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12+\n\x05upper\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12,\n\x06middle\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue2\xc9\x01\n\x18IntradayBollingerService\x12\xac\x01\n\x11IntradayBollinger\x12J.systemathics.apis.services.intraday_analytics.v1.IntradayBollingerRequest\x1aK.systemathics.apis.services.intraday_analytics.v1.IntradayBollingerResponseb\x06proto3')



_INTRADAYBOLLINGERREQUEST = DESCRIPTOR.message_types_by_name['IntradayBollingerRequest']
_INTRADAYBOLLINGERRESPONSE = DESCRIPTOR.message_types_by_name['IntradayBollingerResponse']
_INTRADAYBOLLINGERDATA = DESCRIPTOR.message_types_by_name['IntradayBollingerData']
IntradayBollingerRequest = _reflection.GeneratedProtocolMessageType('IntradayBollingerRequest', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYBOLLINGERREQUEST,
  '__module__' : 'systemathics.apis.services.intraday_analytics.v1.intraday_bollinger_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday_analytics.v1.IntradayBollingerRequest)
  })
_sym_db.RegisterMessage(IntradayBollingerRequest)

IntradayBollingerResponse = _reflection.GeneratedProtocolMessageType('IntradayBollingerResponse', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYBOLLINGERRESPONSE,
  '__module__' : 'systemathics.apis.services.intraday_analytics.v1.intraday_bollinger_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday_analytics.v1.IntradayBollingerResponse)
  })
_sym_db.RegisterMessage(IntradayBollingerResponse)

IntradayBollingerData = _reflection.GeneratedProtocolMessageType('IntradayBollingerData', (_message.Message,), {
  'DESCRIPTOR' : _INTRADAYBOLLINGERDATA,
  '__module__' : 'systemathics.apis.services.intraday_analytics.v1.intraday_bollinger_pb2'
  # @@protoc_insertion_point(class_scope:systemathics.apis.services.intraday_analytics.v1.IntradayBollingerData)
  })
_sym_db.RegisterMessage(IntradayBollingerData)

_INTRADAYBOLLINGERSERVICE = DESCRIPTOR.services_by_name['IntradayBollingerService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INTRADAYBOLLINGERREQUEST._serialized_start=345
  _INTRADAYBOLLINGERREQUEST._serialized_end=622
  _INTRADAYBOLLINGERRESPONSE._serialized_start=624
  _INTRADAYBOLLINGERRESPONSE._serialized_end=738
  _INTRADAYBOLLINGERDATA._serialized_start=741
  _INTRADAYBOLLINGERDATA._serialized_end=963
  _INTRADAYBOLLINGERSERVICE._serialized_start=966
  _INTRADAYBOLLINGERSERVICE._serialized_end=1167
# @@protoc_insertion_point(module_scope)
